@@ -21,8 +21,6 @@
 #include "config.h"
 #include "ephy-web-overview-model.h"
 
-#include <libsoup/soup.h>
-
 struct _EphyWebOverviewModel {
   GObject parent_instance;
 
@@ -131,6 +129,7 @@ ephy_web_overview_model_notify_urls_changed (EphyWebOverviewModel *model)
       if (!urls)
         urls = ephy_web_overview_model_urls_to_js_value (model, jsc_value_get_context (value));
       ret = jsc_value_function_call (value, G_TYPE_PTR_ARRAY, urls, G_TYPE_NONE);
+      (void)ret;
     }
   }
 }
@@ -150,8 +149,10 @@ ephy_web_overview_model_notify_thumbnail_changed (EphyWebOverviewModel *model,
 
     value = jsc_weak_value_get_value (JSC_WEAK_VALUE (key));
     if (value) {
-      if (jsc_value_is_function (value))
+      if (jsc_value_is_function (value)) {
         ret = jsc_value_function_call (value, G_TYPE_STRING, url, G_TYPE_STRING, path, G_TYPE_NONE);
+        (void)ret;
+      }
     }
   }
 }
@@ -171,8 +172,10 @@ ephy_web_overview_model_notify_title_changed (EphyWebOverviewModel *model,
 
     value = jsc_weak_value_get_value (JSC_WEAK_VALUE (key));
     if (value) {
-      if (jsc_value_is_function (value))
+      if (jsc_value_is_function (value)) {
         ret = jsc_value_function_call (value, G_TYPE_STRING, url, G_TYPE_STRING, title, G_TYPE_NONE);
+        (void)ret;
+      }
     }
   }
 }
@@ -281,17 +284,17 @@ ephy_web_overview_model_delete_host (EphyWebOverviewModel *model,
   l = model->items;
   while (l) {
     EphyWebOverviewModelItem *item = (EphyWebOverviewModelItem *)l->data;
-    SoupURI *uri = soup_uri_new (item->url);
+    g_autoptr (GUri) uri = NULL;
     GList *next = l->next;
 
-    if (g_strcmp0 (soup_uri_get_host (uri), host) == 0) {
+    uri = g_uri_parse (item->url, G_URI_FLAGS_NONE, NULL);
+    if (g_strcmp0 (g_uri_get_host (uri), host) == 0) {
       changed = TRUE;
 
       ephy_web_overview_model_item_free (item);
       model->items = g_list_delete_link (model->items, l);
     }
 
-    soup_uri_free (uri);
     l = next;
   }
 
