@@ -26,7 +26,6 @@
 #define GCR_API_SUBJECT_TO_CHANGE
 #include <gcr/gcr.h>
 #include <glib/gi18n.h>
-#include <libsoup/soup.h>
 
 /**
  * SECTION:ephy-certificate-dialog
@@ -63,11 +62,10 @@ static void
 ephy_certificate_dialog_set_address (EphyCertificateDialog *dialog,
                                      const char            *address)
 {
-  SoupURI *uri;
+  g_autoptr (GUri) uri = NULL;
 
-  uri = soup_uri_new (address);
-  gtk_window_set_title (GTK_WINDOW (dialog), uri->host);
-  soup_uri_free (uri);
+  uri = g_uri_parse (address, G_URI_FLAGS_NONE, NULL);
+  gtk_window_set_title (GTK_WINDOW (dialog), g_uri_get_host (uri));
 }
 
 static void
@@ -328,16 +326,16 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
 }
 
 GtkWidget *
-ephy_certificate_dialog_new (GtkWindow           *parent,
-                             const char          *address,
-                             GTlsCertificate     *certificate,
-                             GTlsCertificateFlags tls_errors,
-                             EphySecurityLevel    security_level)
+ephy_certificate_dialog_new (GtkWindow            *parent,
+                             const char           *address,
+                             GTlsCertificate      *certificate,
+                             GTlsCertificateFlags  tls_errors,
+                             EphySecurityLevel     security_level)
 {
   GtkWidget *dialog;
 
-  g_return_val_if_fail (address != NULL, NULL);
-  g_return_val_if_fail (G_IS_TLS_CERTIFICATE (certificate), NULL);
+  g_assert (address != NULL);
+  g_assert (G_IS_TLS_CERTIFICATE (certificate));
 
   dialog = GTK_WIDGET (g_object_new (EPHY_TYPE_CERTIFICATE_DIALOG,
                                      "address", address,
