@@ -133,70 +133,45 @@ combo_box_changed_cb (GtkComboBox *combo_box,
     gtk_button_set_label (button, _("I_mport"));
 }
 
-static gchar *
-get_path (GIOChannel *channel)
-{
-  gchar *line;
-  const gchar *path;
-  gchar *result;
-  gsize length;
-
-  do {
-    g_io_channel_read_line (channel, &line, &length, NULL, NULL);
-
-    if (g_str_has_prefix (line, "Path")) {
-      path = line;
-
-      /* Extract value (e.g. Path=Value\n -> Value) */
-      path = strchr (path, '=');
-      path++;
-      ((gchar *)path)[strcspn (path, "\n")] = '\0';
-      result = g_strdup (path);
-
-      g_free (line);
-      return result;
-    }
-
-    g_free (line);
-    /* Until '\n' */
-  } while (length != 1);
-
-  return NULL;
-}
-
 static GSList *
 get_firefox_profiles (void)
 {
-  GIOChannel *channel;
+  GKeyFile *keyfile;
   GSList *profiles = NULL;
-  gchar *filename;
-  gchar *line;
-  gchar *profile;
-  int count = 0;
-  gsize length;
+  g_autofree gchar *filename = NULL;
+  g_autoptr (GError) error = NULL;
+  g_auto (GStrv) groups = NULL;
+  int i = 0;
 
   filename = g_build_filename (g_get_home_dir (),
                                FIREFOX_PROFILES_DIR,
                                FIREFOX_PROFILES_FILE,
                                NULL);
-  channel = g_io_channel_new_file (filename, "r", NULL);
-  g_free (filename);
+  keyfile = g_key_file_new ();
+  g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, &error);
+  if (error) {
+    g_warning ("Failed to load %s: %s", filename, error->message);
 
-  if (channel) {
-    do {
-      g_io_channel_read_line (channel, &line, &length, NULL, NULL);
+    return NULL;
+  }
 
-      profile = g_strdup_printf ("[Profile%d]\n", count);
-      if (g_strcmp0 (line, profile) == 0) {
-        profiles = g_slist_append (profiles, get_path (channel));
+  groups = g_key_file_get_groups (keyfile, NULL);
+  while (groups[i]) {
+    const char *group = groups[i++];
+    char *path;
 
-        count++;
-      }
-      g_free (profile);
-      g_free (line);
-    } while (length != 0);
+    if (!g_str_has_prefix (group, "Profile"))
+      continue;
 
-    g_io_channel_unref (channel);
+    path = g_key_file_get_string (keyfile, group, "Path", &error);
+    if (error) {
+      g_warning ("Failed to parse profile %s in %s: %s",
+                 groups[i], filename, error->message);
+
+      continue;
+    }
+
+    profiles = g_slist_append (profiles, path);
   }
 
   return profiles;
@@ -210,7 +185,7 @@ firefox_exists (void)
 
   firefox_profiles = get_firefox_profiles ();
   has_firefox_profile = g_slist_length (firefox_profiles) > 0;
-  g_slist_free (firefox_profiles);
+  g_slist_free_full (firefox_profiles, g_free);
 
   return has_firefox_profile;
 }
@@ -295,7 +270,7 @@ show_profile_selector (GtkWidget *parent,
     GtkWidget *label;
 
     label = gtk_label_new (strchr (profile, '.') + 1);
-    g_object_set_data (G_OBJECT (label), "profile_path", g_strdup (profile));
+    g_object_set_data_full (G_OBJECT (label), "profile_path", g_strdup (profile), g_free);
     gtk_widget_set_margin_top (label, 6);
     gtk_widget_set_margin_bottom (label, 6);
     gtk_list_box_insert (GTK_LIST_BOX (list_box), label, -1);
@@ -311,7 +286,7 @@ show_profile_selector (GtkWidget *parent,
 
     row = gtk_list_box_get_selected_row (GTK_LIST_BOX (list_box));
     row_widget = gtk_bin_get_child (GTK_BIN (row));
-    selected_profile = g_object_get_data (G_OBJECT (row_widget), "profile_path");
+    selected_profile = g_object_steal_data (G_OBJECT (row_widget), "profile_path");
   }
   gtk_widget_destroy (selector);
 
